@@ -1,5 +1,6 @@
 package Lingua::Stem::Snowball;
 use strict;
+use warnings;
 
 use Carp;
 use Exporter;
@@ -13,16 +14,20 @@ use vars qw(
     %instance_vars
 );
 
-$VERSION = '0.94';
+$VERSION = '0.95';
 
-@ISA         = qw( Exporter );
+@ISA         = qw( Exporter DynaLoader );
 %EXPORT_TAGS = ( 'all' => [qw( stemmers stem )] );
 @EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
 
-require XSLoader;
-XSLoader::load( 'Lingua::Stem::Snowball', $VERSION );
+require DynaLoader;
+__PACKAGE__->bootstrap($VERSION);
 
-# a shared home for the actual struct sb_stemmer C modules.
+# Ensure that C symbols are exported so that other shared libaries (e.g.
+# KinoSearch) can use them.  See Dynaloader docs.
+sub dl_load_flags {0x01}
+
+# A shared home for the actual struct sb_stemmer C modules.
 $stemmifier = Lingua::Stem::Snowball::Stemmifier->new;
 
 %instance_vars = (
@@ -35,12 +40,16 @@ $stemmifier = Lingua::Stem::Snowball::Stemmifier->new;
 
 sub new {
     my $class = shift;
-    my $self  = bless { %instance_vars, @_ }, ref($class) || $class;
+    my $self = bless { %instance_vars, @_ }, ref($class) || $class;
 
-    # validate lang, validate/guess encoding, and get an sb_stemmer
+    # Validate lang, validate/guess encoding, and get an sb_stemmer.
     $self->lang( $self->{lang} );
     if ( !defined $self->{encoding} ) {
-        $self->{encoding} = $self->{lang} eq 'ru' ? 'KOI8-R' : 'ISO-8859-1';
+        $self->{encoding}
+            = $self->{lang} eq 'ru' ? 'KOI8-R'
+            : $self->{lang} eq 'ro' ? 'ISO-8859-2'
+            : $self->{lang} eq 'tr' ? 'UTF-8'
+            :                         'ISO-8859-1';
     }
     $self->_derive_stemmer;
 
@@ -50,7 +59,7 @@ sub new {
 sub stem {
     my ( $self, $lang, $words, $locale, $is_stemmed );
 
-    # support lots of DWIMmery
+    # Support lots of DWIMmery.
     if ( UNIVERSAL::isa( $_[0], 'HASH' ) ) {
         ( $self, $words, $is_stemmed ) = @_;
     }
@@ -59,18 +68,18 @@ sub stem {
         $self = __PACKAGE__->new( lang => $lang );
     }
 
-    # bail if we don't have a valid lang
+    # Bail if we don't have a valid lang.
     return undef unless $self->{lang};
 
-    # bail if there's no input
+    # Bail if there's no input.
     return undef unless ( ref($words) or length($words) );
 
-    # duplicate the input array and transform it into an array of stems
+    # Duplicate the input array and transform it into an array of stems.
     $words = ref($words) ? $words : [$words];
     my @stems = map {lc} @$words;
     $self->stem_in_place( \@stems );
 
-    # determine whether any stemming took place, if requested
+    # Determine whether any stemming took place, if requested.
     if ( ref($is_stemmed) ) {
         $$is_stemmed = 0;
         if ( $self->{stemmer_id} == -1 ) {
@@ -95,7 +104,7 @@ sub lang {
         $lang = $lang eq 'dk' ? 'nl' : $lang;    # backwards compat
         if ( _validate_language($lang) ) {
             $self->{lang} = $lang;
-            # force stemmer_id regen at next call to stem_in_place
+            # Force stemmer_id regen at next call to stem_in_place().
             $self->{stemmer_id} = -1;
         }
         else {
@@ -109,15 +118,15 @@ sub encoding {
     my ( $self, $encoding ) = @_;
     if ( defined $encoding ) {
         croak("Invalid value for encoding: '$encoding'")
-            unless $encoding =~ /^(?:UTF-8|KOI8-R|ISO-8859-1)$/;
+            unless $encoding =~ /^(?:UTF-8|KOI8-R|ISO-8859-[12])$/;
         $self->{encoding} = $encoding;
-        # force stemmer_id regen at next call to stem_in_place
+        # Force stemmer_id regen at next call to stem_in_place().
         $self->{stemmer_id} = -1;
     }
     return $self->{encoding};
 }
 
-# deprecated, has no effect on stemming behavior
+# Deprecated, has no effect on stemming behavior.
 sub strip_apostrophes {
     my ( $self, $boolean ) = @_;
     if ( defined $boolean ) {
@@ -126,7 +135,7 @@ sub strip_apostrophes {
     return $self->{strip_apostrophes};
 }
 
-# deprecated, has no effect on stemming behavior
+# Deprecated, has no effect on stemming behavior.
 sub locale {
     my ( $self, $locale ) = @_;
     if ($locale) {
@@ -151,24 +160,24 @@ Lingua::Stem::Snowball - Perl interface to Snowball stemmers.
     my $stemmer = Lingua::Stem::Snowball->new( lang => 'en' );
     $stemmer->stem_in_place( \@words ); # qw( hors hoov )
 
-    # plain interface:
+    # Functional interface:
     my @stems = stem( 'en', \@words );
 
 =head1 DESCRIPTION
 
-Stemming reduces related words to a common root form. For instance, "horse",
+Stemming reduces related words to a common root form -- for instance, "horse",
 "horses", and "horsing" all become "hors".  Most commonly, stemming is
 deployed as part of a search application, allowing searches for a given term
 to match documents which contain other forms of that term.
 
-This module is very similar to L<Lingua::Stem|Lingua::Stem> -- however,
-Lingua::Stem is pure Perl, while Lingua::Stem::Snowball is an XS module which
-provides a Perl interface to the C version of the Snowball stemmers.
+This module is very similar to L<Lingua::Stem> -- however, Lingua::Stem is
+pure Perl, while Lingua::Stem::Snowball is an XS module which provides a Perl
+interface to the C version of the Snowball stemmers.
 (L<http://snowball.tartarus.org>).  
 
 =head2 Supported Languages
 
-The following stemmers are available (as of Lingua::Stem::Snowball 0.94):
+The following stemmers are available (as of Lingua::Stem::Snowball 0.95):
 
     |-----------------------------------------------------------|
     | Language   | ISO code | default encoding | also available |
@@ -179,12 +188,15 @@ The following stemmers are available (as of Lingua::Stem::Snowball 0.94):
     | Finnish    | fi       | ISO-8859-1       | UTF-8          | 
     | French     | fr       | ISO-8859-1       | UTF-8          |
     | German     | de       | ISO-8859-1       | UTF-8          | 
+    | Hungarian  | hu       | ISO-8859-1       | UTF-8          | 
     | Italian    | it       | ISO-8859-1       | UTF-8          | 
     | Norwegian  | no       | ISO-8859-1       | UTF-8          | 
     | Portuguese | pt       | ISO-8859-1       | UTF-8          | 
+    | Romanian   | ro       | ISO-8859-2       | UTF-8          | 
+    | Russian    | ru       | KOI8-R           | UTF-8          | 
     | Spanish    | es       | ISO-8859-1       | UTF-8          | 
     | Swedish    | sv       | ISO-8859-1       | UTF-8          | 
-    | Russian    | ru       | KOI8-R           | UTF-8          | 
+    | Turkish    | tr       | UTF-8            |                | 
     |-----------------------------------------------------------|
 
 =head2 Benchmarks
@@ -193,7 +205,7 @@ Here is a comparison of Lingua::Stem::Snowball and Lingua::Stem, using The
 Works of Edgar Allen Poe, volumes 1-5 (via Project Gutenberg) as source
 material.  It was produced on a 3.2GHz Pentium 4 running FreeBSD 5.3 and Perl
 5.8.7.  (The benchmarking script is included in this distribution:
-bin/benchmark_stemmers.plx.)
+devel/benchmark_stemmers.plx.)
 
     |--------------------------------------------------------------------|
     | total words: 454285 | unique words: 22748                          |
@@ -240,7 +252,7 @@ change: stem() will always return undef, and stem_in_place() will be a no-op.
 =head2 stem
 
     @stemmed = $stemmer->stem( WORDS, [IS_STEMMED] );
-    @stemmed = stem( ISO_CODE, WORDS, [LOCALE, IS_STEMMED] );
+    @stemmed = stem( ISO_CODE, WORDS, [LOCALE], [IS_STEMMED] );
 
 Return lowercased and stemmed output.  WORDS may be either an array of words
 or a single scalar word.  
@@ -312,9 +324,10 @@ Currently maintained by Marvin Humphrey E<lt>marvin at rectangular dot
 comE<gt>.  Previously maintained by Fabien Potencier E<lt>fabpot at cpan dot
 orgE<gt>.  
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2004-2006
+Perl bindings copyright 2004-2008 by Marvin Humphrey, Fabien Potencier, Oleg
+Bartunov and Teodor Sigaev.
 
 This software may be freely copied and distributed under the same
 terms and conditions as Perl.
@@ -326,5 +339,3 @@ Snowball files and stemmers are covered by the BSD license.
 L<http://snowball.tartarus.org>, L<Lingua::Stem|Lingua::Stem>.
 
 =cut
-
-
